C
Code by Michael Kupferschmid
C
C     This program compresses vertical-axis data in a file of
C     [x,y] or [x,y,z] coordinates for plotting.
C
C     usage: compress [k]
C     where k is the contrast level, defaulting to 0
C
C     unit  use
C     ----  --------------------------------
C       0   error and informational messages
C       1   input
C       2   output
C
C     variable  meaning
C     --------  -------
C     ARG       command-line parameter as characters
C     DABS      Fortran function returns |REAL*8|
C     DATA      a pair or triple of data values
C     DLOG      Fortran function returns natural log of a REAL*8
C     DMAX1     Fortran function returns larger of REAL*8s
C     DMIN1     Fortran function returns smaller of REAL*8s
C     DSIGN     Fortran function transfers sign of a REAL*8
C     DTB       routine converts numerals to an INTEGER*4
C     EXIT      system routine stops program with a return code
C     FILNAM    input then output filename if read from keyboard
C     FRDR8V    routine reads unknown-length line of REAL*8 values
C     GETARG    system routine gets one command-line parameter
C     GETFIL    routine attaches a file for a given purpose
C     I         index on lines written
C     IA        index on run-line parameters
C     IARGC     system function for number of command-line arguments
C     J         index on the DATA elements of a line
C     K         contrast reduction, defaults to 0
C     L         length of FILNAM
C     LENGTH    function returns index of last nonblank in a string
C     LNA       factor makes VMAX map to +1 or -1
C     LNUM      number of the input line just read
C     LVM       factor makes VMIN map to 0
C     N         number of data items on a line of the input file
C     NARG      number of command-line parameters found
C     NPREV     number of data items on previous line of input file
C     PACKL     routine removes leading and embedded blanks
C     RC        return codes; 0 => all went well
C     SHIFTR    routine makes string flush right in its length
C     STEEP     steepness of compression, defaults to 1
C     V         a scaled data value
C     VMAX      largest absolute vertical-axis value found
C     VMIN      smallest absolute vertical-axis value found
C
C     prepare to control compression
      CHARACTER*3 ARG
      REAL*8 STEEP,LNA,LVM
C
C     prepare to process data
      REAL*8 DATA(3),V,VMAX/0.D0/,VMIN/Z'7FEFFFFFFFFFFFFF'/
      INTEGER*4 RC
C
C     prepare to process filenames
      CHARACTER*24 FILNAM
C
C ------------------------------------------------------------------
C
C     set contrast based on numerical command-line parameter, if any
      K=0
      NARG=IARGC()
      IF(NARG.EQ.0) GO TO 1
      DO 2 IA=1,NARG
           CALL GETARG(IA,ARG)
           CALL SHIFTR(ARG,3)
           CALL DTB(ARG,3, K,RC)
           IF(RC.EQ.0) GO TO 1
    2 CONTINUE
    1 STEEP=10.D0**K
C
C     attach the input file
      CALL GETFIL('data',4,' ',0,1,1)
      LNUM=0
      NPREV=0
C
C     read the file once to find the extreme vertical-axis values
    4 CALL FRDR8V(1,DATA,3, N,RC)
        IF(RC.EQ.1) GO TO 3
        LNUM=LNUM+1
        IF(RC.EQ.5) GO TO 4
        IF(RC.NE.0) THEN
           WRITE(0,901) LNUM
  901      FORMAT('read error at line',I10)
           CALL EXIT(1)
        ENDIF
        IF(LNUM.EQ.1) THEN
           IF(N.EQ.2 .OR. N.EQ.3) THEN
              NPREV=N
           ELSE
              WRITE(0,902) N
  902         FORMAT('need 2 or 3 columns, not',I2)
              CALL EXIT(2)
           ENDIF
        ENDIF
        IF(N.NE.NPREV) THEN
           WRITE(0,903) LNUM
  903      FORMAT('change in column count at line ',I10)
           CALL EXIT(3)
        ENDIF
C
C       find absolutely biggest and smallest vertical-axis values
        VMAX=DMAX1(VMAX,DABS(DATA(N)))
        VMIN=DMIN1(VMIN,DABS(DATA(N)))
      GO TO 4
    3 N=NPREV
C
C     data absent or having zero range cannot be compressed
      IF(N.EQ.0 .OR. VMAX.EQ.VMIN) THEN
         WRITE(0,904)
  904    FORMAT('this data has zero range')
         CALL EXIT(4)
      ENDIF
C
C     attach a file for the output
      INQUIRE(UNIT=1,NAME=FILNAM)
      FILNAM(21:24)='.cpr'
      CALL PACKL(FILNAM,24)
      L=LENGTH(FILNAM,24)
      CALL GETFIL('output',6,FILNAM,L,2,2)
C
C     reread the file and write a compressed version
      REWIND(UNIT=1)
      LVM=DLOG(1.D0+STEEP*VMIN/VMAX)
      LNA=DLOG(1.D0+STEEP)-LVM
      DO 5 I=1,LNUM
    6      CALL FRDR8V(1,DATA,3, N,RC)
           IF(RC.EQ.5) GO TO 6
           V=STEEP*DATA(N)/VMAX
           DATA(N)=DSIGN(DABS((DLOG(1.D0+DABS(V))-LVM)/LNA),V)
           WRITE(2,905) (DATA(J),J=1,N)
  905      FORMAT(3(1X,1PE13.6))
    5 CONTINUE
      CALL EXIT(0)
      END
