C
Code by Michael Kupferschmid
C
C     This program reads data from standard-in and writes one or
C     more descriptive statistics to standard-out.  These are the
C     statistics that can be requested:
C
C       j    statistic S(j)
C     -----  ----------------------
C       1    number of observations
C       2    lowest value
C       3    highest value
C       4    sum
C       5    mean
C       6    median
C       7    standard deviation
C
C     variable  meaning
C     --------  -------
C     ARG       a command-line argument as characters
C     DFLOAT    Fortran function returns REAL*8 for INTEGER*4
C     DMAX1     Fortran function returns larger of REAL*8s
C     DMIN1     Fortran function returns smaller of REAL*8s
C     DSQRT     Fortran function returns square root of a REAL*8
C     DTB       routine converts numerals to an INTEGER*4
C     EXIT      system routine stops program with a return code
C     GETARG    system routine gets one command-line parameter
C     I         index on observations
C     IA        index on command-line parameters
C     IARGC     system function for number of command-line arguments
C     IS        numbers of statistics requested
C     MED       T => compute median
C     MOD       Fortran function for remainder of INTEGER*4 division
C     N         number of observations read
C     NARG      number of command-line parameters
C     NMAX      limit on observations to store
C     RC        return code from DTB; 0 => all went well
C     S         statistics; see table above
C     SHIFTR    routine makes string flush right in its length
C     SORTR8    routine sorts a REAL*8 vector in ascending order
C     SSQ       sum of squares in calculation of standard deviation
C     STD       T => compute standard deviation
C     X         an observation
C     Y         observations stored for median or standard deviation
C
C     prepare to interpret command-line parameters
      CHARACTER*2 ARG
      INTEGER*4 IS(7),RC
      LOGICAL*4 MED,STD
C
C     prepare to process data
      PARAMETER(NMAX=1000)
      REAL*8 Y(NMAX),X,SSQ
      REAL*8 S(7)/0.D0,Z'7FEFFFFFFFFFFFFF',Z'FFFFFFFFFFFFFFFF',
     ;            0.D0,0.D0,0.D0,0.D0/
C
C ------------------------------------------------------------------
C
C     figure out which statistics are to be reported in what order
      NARG=IARGC()
      IF(NARG.LE.0 .OR. NARG.GT.7) THEN
C        explain how to use the program
         WRITE(0,900)
  900    FORMAT('usage: cat data | statistic a b ...'/
     ;          '       where a b ... are numbers of statistics'/
     ;          '       as defined in the man page for the program')
         CALL EXIT(1)
      ELSE
         MED=.FALSE.
         STD=.FALSE.
         DO 1 IA=1,NARG
              CALL GETARG(IA,ARG)
              CALL SHIFTR(ARG,2)
              CALL DTB(ARG,2, IS(IA),RC)
              IF(RC.NE.0 .OR. IS(IA).LE.0 .OR. IS(IA).GT.7) THEN
                 WRITE(0,991) IA
  991            FORMAT('parameter ',I1,' is bad')
                 CALL EXIT(2)
              ENDIF
C             these statistics require observations to be stored
              IF(IS(IA).EQ.6) MED=.TRUE.
              IF(IS(IA).EQ.7) STD=.TRUE.
    1    CONTINUE
      ENDIF
      N=0
C
C     get the next input value and accumulate some statistics
    4 READ(5,*,END=2,ERR=3) X
      N=N+1
      S(2)=DMIN1(S(2),X)
      S(3)=DMAX1(S(3),X)
      S(4)=S(4)+X
      IF(MED .OR. STD) THEN
         IF(N.LE.NMAX) THEN
            Y(N)=X
         ELSE
            WRITE(0,992) NMAX
  992       FORMAT('more than ',I4,' data points')
            CALL EXIT(3)
         ENDIF
      ENDIF
      GO TO 4
C
C     compute other statistics
    2 IF(N.EQ.0) THEN
         WRITE(0,993)
  993    FORMAT('no data')
         CALL EXIT(4)
      ENDIF
      S(1)=DFLOAT(N)
      S(5)=S(4)/DFLOAT(N)
      IF(MED) THEN
         CALL SORTR8(N,Y)
         IF(MOD(N,2).EQ.1) THEN
            S(6)=Y(N/2)
         ELSE
            S(6)=0.5D0*(Y(N/2)+Y(N/2+1))
         ENDIF
      ENDIF
      IF(STD) THEN
         SSQ=0.D0
         DO 5 I=1,N
              SSQ=SSQ+(Y(I)-S(5))**2
    5    CONTINUE
         S(7)=DSQRT(SSQ/DFLOAT(N-1))
      ENDIF
C
C     write the requested statistics
      WRITE(6,901) (S(IS(IA)),IA=1,NARG)
  901 FORMAT(7(1X,1PE13.6))
      CALL EXIT(0)
C
C     report bad data
    3 WRITE(0,994) LNUM
  994 FORMAT('read error at line ',I4)
      CALL EXIT(5)
      END
